"""One of the example files showing relative ordering between modules.
See https://pytest-order.readthedocs.io/en/stable/#referencing-of-tests-in-other-classes-or-modules  # noqa: E501
"""


def test_1():
    pass


def test_2():
    pass



import pytest


@pytest.mark.order(1)
class Test1:
    def test_1(self):
        assert True

    def test_2(self):
        assert True


@pytest.mark.order(0)
class Test2:
    def test_1(self):
        assert True

    def test_2(self):
        assert True



import pytest


@pytest.fixture
def sparse_ordering(ignore_settings):
    ignore_settings.return_value.sparse_ordering = True
    yield


@pytest.fixture(scope="module")
def first_test():
    yield (
        """
        import pytest

        def test_1(): pass

        @pytest.mark.order("first")
        def test_2(): pass
        """
    )


def test_first_default(first_test, item_names_for):
    assert item_names_for(first_test) == ["test_2", "test_1"]


def test_first_sparse(first_test, item_names_for, sparse_ordering):
    assert item_names_for(first_test) == ["test_2", "test_1"]


@pytest.fixture(scope="module")
def second_test():
    yield (
        """
        import pytest

        def test_1(): pass
        def test_2(): pass
        def test_3(): pass
        def test_4(): pass

        @pytest.mark.order("second")
        def test_5(): pass
        """
    )


def test_second_default(second_test, item_names_for):
    assert item_names_for(second_test) == [
        "test_5",
        "test_1",
        "test_2",
        "test_3",
        "test_4",
    ]


def test_second_sparse(second_test, item_names_for, sparse_ordering):
    assert item_names_for(second_test) == [
        "test_1",
        "test_5",
        "test_2",
        "test_3",
        "test_4",
    ]


@pytest.fixture(scope="module")
def third_test():
    yield (
        """
        import pytest

        def test_1(): pass
        def test_2(): pass
        def test_3(): pass

        @pytest.mark.order("third")
        def test_4(): pass

        def test_5(): pass
        """
    )


def test_third_default(third_test, item_names_for):
    assert item_names_for(third_test) == [
        "test_4",
        "test_1",
        "test_2",
        "test_3",
        "test_5",
    ]


def test_third_sparse(third_test, item_names_for, sparse_ordering):
    assert item_names_for(third_test) == [
        "test_1",
        "test_2",
        "test_4",
        "test_3",
        "test_5",
    ]


@pytest.fixture(scope="module")
def second_to_last_test():
    yield (
        """
        import pytest

        def test_1(): pass

        @pytest.mark.order("second_to_last")
        def test_2(): pass

        def test_3(): pass
        def test_4(): pass
        def test_5(): pass
        """
    )


def test_second_to_last_default(second_to_last_test, item_names_for):
    assert item_names_for(second_to_last_test) == [
        "test_1",
        "test_3",
        "test_4",
        "test_5",
        "test_2",
    ]


def test_second_to_last_sparse(second_to_last_test, item_names_for, sparse_ordering):
    assert item_names_for(second_to_last_test) == [
        "test_1",
        "test_3",
        "test_4",
        "test_2",
        "test_5",
    ]


@pytest.fixture(scope="module")
def last_test():
    yield (
        """
        import pytest

        @pytest.mark.order("last")
        def test_1(): pass

        def test_2(): pass
        """
    )


def test_last_default(last_test, item_names_for):
    assert item_names_for(last_test) == ["test_2", "test_1"]


def test_last_sparse(last_test, item_names_for, sparse_ordering):
    assert item_names_for(last_test) == ["test_2", "test_1"]


@pytest.fixture(scope="module")
def first_last_test():
    yield (
        """
        import pytest

        @pytest.mark.order("last")
        def test_1(): pass

        @pytest.mark.order("first")
        def test_2(): pass

        def test_3(): pass
        """
    )


def test_first_last_default(first_last_test, item_names_for):
    assert item_names_for(first_last_test) == ["test_2", "test_3", "test_1"]


def test_first_last_sparse(first_last_test, item_names_for, sparse_ordering):
    assert item_names_for(first_last_test) == ["test_2", "test_3", "test_1"]


@pytest.fixture(scope="module")
def duplicate_numbers_test():
    yield (
        """
        import pytest

        @pytest.mark.order(1)
        def test_1(): pass

        @pytest.mark.order(1)
        def test_2(): pass

        def test_3(): pass

        def test_4(): pass

        @pytest.mark.order(4)
        def test_5(): pass
        """
    )


def test_duplicate_numbers_default(duplicate_numbers_test, item_names_for):
    assert item_names_for(duplicate_numbers_test) == [
        "test_1",
        "test_2",
        "test_5",
        "test_3",
        "test_4",
    ]


def test_duplicate_numbers_sparse(
    duplicate_numbers_test, item_names_for, sparse_ordering
):
    assert item_names_for(duplicate_numbers_test) == [
        "test_3",
        "test_1",
        "test_2",
        "test_4",
        "test_5",
    ]


@pytest.fixture(scope="module")
def end_items_test():
    yield (
        """
        import pytest

        @pytest.mark.order(-2)
        def test_1(): pass

        @pytest.mark.order(-4)
        def test_2(): pass

        def test_3(): pass

        def test_4(): pass

        def test_5(): pass
        """
    )


def test_end_items_default(end_items_test, item_names_for):
    assert item_names_for(end_items_test) == [
        "test_3",
        "test_4",
        "test_5",
        "test_2",
        "test_1",
    ]


def test_end_items_sparse(end_items_test, item_names_for, sparse_ordering):
    assert item_names_for(end_items_test) == [
        "test_3",
        "test_2",
        "test_4",
        "test_1",
        "test_5",
    ]



"""One of the example files showing relative ordering between modules.
See https://pytest-order.readthedocs.io/en/stable/#referencing-of-tests-in-other-classes-or-modules  # noqa: E501
"""


class TestA:
    def test_a(self):
        assert True

    def test_b(self):
        assert True



"""Example showing ordering relative to tests in other modules.
See https://pytest-order.readthedocs.io/en/stable/#referencing-of-tests-in-other-classes-or-modules  # noqa: E501
"""
import pytest


@pytest.mark.order(after="test_module_a.py::TestA::test_a")
def test_a():
    assert True


@pytest.mark.order(before="test_module_c.py::test_submodule.test_2")
def test_b():
    assert True



import time

from pytest_order.sorter import Sorter


class TimedSorter(Sorter):
    elapsed = 0.0
    nr_marks = 1000

    def sort_items(self):
        self.__class__.elapsed = 0.0
        start_time = time.time()
        items = super().sort_items()
        self.__class__.elapsed = (time.time() - start_time) / self.nr_marks * 1000
        print("\nTime per test: {:.3f} ms".format(self.__class__.elapsed))
        return items



"""Shows how to use both positive and negative numbers for sorting,
and the use of the long form (index attribute) and short form (number only).
See https://pytest-order.readthedocs.io/en/stable/#order-by-index
"""
import pytest


@pytest.mark.order(-2)
def test_three():
    assert True


@pytest.mark.order(index=-1)
def test_four():
    assert True


@pytest.mark.order(index=2)
def test_two():
    assert True


@pytest.mark.order(1)
def test_one():
    assert True



import pytest


@pytest.fixture(scope="module")
def marker_test_file():
    yield (
        """
        import pytest

        @pytest.mark.my3
        def test_a():
            pass

        @pytest.mark.my1
        def test_b():
            pass

        @pytest.mark.my2
        def test_c():
            pass
        """
    )


@pytest.fixture
def marker_test(test_path, marker_test_file):
    test_path.makepyfile(test_marker=marker_test_file)
    yield test_path


def test_no_ordering(marker_test_file, item_names_for):
    assert item_names_for(marker_test_file) == ["test_a", "test_b", "test_c"]


def test_order_with_marker_prefix(marker_test):
    result = marker_test.runpytest("-v", "--order-marker-prefix=my")
    result.assert_outcomes(passed=3, skipped=0)
    result.stdout.fnmatch_lines(
        [
            "test_marker.py::test_b PASSED",
            "test_marker.py::test_c PASSED",
            "test_marker.py::test_a PASSED",
        ]
    )


def test_order_with_marker_prefix_filtered(marker_test):
    result = marker_test.runpytest("-v", "--order-marker-prefix=my", "-m", "my2 or my3")
    result.assert_outcomes(passed=2, skipped=0)
    result.stdout.fnmatch_lines(
        [
            "test_marker.py::test_c PASSED",
            "test_marker.py::test_a PASSED",
        ]
    )


def test_no_ordering_with_incorrect_marker_prefix(marker_test):
    result = marker_test.runpytest("-v", "--order-marker-prefix=mi")
    result.assert_outcomes(passed=3, skipped=0)
    result.stdout.fnmatch_lines(
        [
            "test_marker.py::test_a PASSED",
            "test_marker.py::test_b PASSED",
            "test_marker.py::test_c PASSED",
        ]
    )


def test_no_ordering_with_shorter_marker_prefix(marker_test):
    result = marker_test.runpytest("-v", "--order-marker-prefix=m")
    result.assert_outcomes(passed=3, skipped=0)
    result.stdout.fnmatch_lines(
        [
            "test_marker.py::test_a PASSED",
            "test_marker.py::test_b PASSED",
            "test_marker.py::test_c PASSED",
        ]
    )


def test_marker_prefix_does_not_interfere_with_order_marks(test_path):
    test_path.makepyfile(
        test_marker=(
            """
            import pytest

            @pytest.mark.order(3)
            def test_a():
                pass

            @pytest.mark.order(1)
            def test_b():
                pass

            @pytest.mark.order(2)
            def test_c():
                pass
            """
        )
    )
    result = test_path.runpytest("-v", "--order-marker-prefix=m")
    result.assert_outcomes(passed=3, skipped=0)
    result.stdout.fnmatch_lines(
        [
            "test_marker.py::test_b PASSED",
            "test_marker.py::test_c PASSED",
            "test_marker.py::test_a PASSED",
        ]
    )


def test_mix_marker_prefix_with_order_marks(test_path):
    test_path.makepyfile(
        test_marker=(
            """
            import pytest

            @pytest.mark.order(3)
            def test_a():
                pass

            @pytest.mark.my1
            def test_b():
                pass

            @pytest.mark.my2
            def test_c():
                pass
            """
        )
    )
    result = test_path.runpytest("-v", "--order-marker-prefix=my")
    result.assert_outcomes(passed=3, skipped=0)
    result.stdout.fnmatch_lines(
        [
            "test_marker.py::test_b PASSED",
            "test_marker.py::test_c PASSED",
            "test_marker.py::test_a PASSED",
        ]
    )



import pytest
from pytest_order.settings import Scope

pytest_plugins = ["pytester"]


@pytest.fixture
def item_names_for(testdir):
    def _item_names_for(tests_content):
        def name(item):
            if item.cls:
                return item.cls.__name__ + "::" + item.name
            return item.name

        items = testdir.getitems(tests_content)
        hook = items[0].config.hook
        hook.pytest_collection_modifyitems(
            session=items[0].session, config=items[0].config, items=items
        )

        return [name(item) for item in items]

    return _item_names_for


@pytest.fixture
def test_path(testdir):
    testdir.tmpdir.join("pytest.ini").write("[pytest]\nconsole_output_style = classic")
    yield testdir


@pytest.fixture
def ignore_settings(mocker):
    settings = mocker.patch("pytest_order.sorter.Settings")
    settings.return_value.sparse_ordering = False
    settings.return_value.order_dependencies = False
    settings.return_value.scope = Scope.SESSION
    settings.return_value.group_scope = Scope.SESSION
    settings.return_value.scope_level = 0
    settings.return_value.marker_prefix = None
    yield settings


@pytest.fixture
def order_dependencies(ignore_settings):
    ignore_settings.return_value.order_dependencies = True
    yield



def test_multiple_markers(item_names_for):
    test_content = """
        import pytest

        @pytest.mark.order(2)
        def test_1():
            pass

        @pytest.mark.order(1)
        @pytest.mark.order(3)
        def test_2():
            pass
        """
    assert item_names_for(test_content) == [
        "test_2[index=1]",
        "test_1",
        "test_2[index=3]",
    ]


def test_with_relative_markers(item_names_for):
    test_content = """
        import pytest

        def test_1():
            pass

        @pytest.mark.order(before="test_1")
        @pytest.mark.order(2)
        def test_2():
            pass

        @pytest.mark.order(1)
        @pytest.mark.order(before="test_1")
        def test_3():
            pass

        @pytest.mark.order(-1)
        @pytest.mark.order(-3)
        def test_4():
            pass

        @pytest.mark.order(-2)
        @pytest.mark.order(-4)
        def test_5():
            pass
        """
    assert item_names_for(test_content) == [
        "test_3[index=1]",
        "test_2[index=2]",
        "test_3[before=test_1]",
        "test_2[before=test_1]",
        "test_1",
        "test_5[index=-4]",
        "test_4[index=-3]",
        "test_5[index=-2]",
        "test_4[index=-1]",
    ]


def test_multiple_markers_with_parametrization(item_names_for):
    test_content = """
        import pytest

        @pytest.mark.order(2)
        def test_1():
            pass

        @pytest.mark.parametrize("arg", ["aaaaa", "bbbbb", "ccccc", "ddddd"])
        @pytest.mark.order(1)
        @pytest.mark.order(-1)
        def test_2(arg):
            pass

        @pytest.mark.order(3)
        def test_3():
            pass
        """
    assert item_names_for(test_content) == [
        "test_2[index=1-aaaaa]",
        "test_2[index=1-bbbbb]",
        "test_2[index=1-ccccc]",
        "test_2[index=1-ddddd]",
        "test_1",
        "test_3",
        "test_2[index=-1-aaaaa]",
        "test_2[index=-1-bbbbb]",
        "test_2[index=-1-ccccc]",
        "test_2[index=-1-ddddd]",
    ]


def test_multiple_markers_in_class(item_names_for):
    test_content = """
        import pytest

        class TestA:
            @pytest.mark.order(1)
            @pytest.mark.order(3)
            def test_1_and_3():
                pass
            @pytest.mark.order(-1)
            def test_4():
                pass

        @pytest.mark.order(2)
        def test_2():
            pass
        """
    assert item_names_for(test_content) == [
        "TestA::test_1_and_3[index=1]",
        "test_2",
        "TestA::test_1_and_3[index=3]",
        "TestA::test_4",
    ]



"""Test with ordering (as opposed to test.py) from the quick start chapter.
See https://pytest-order.readthedocs.io/en/stable/#quickstart
"""
import pytest


@pytest.mark.order(2)
def test_foo():
    assert True


@pytest.mark.order(1)
def test_bar():
    assert True



"""Shows how to sort tests relatively to each other, see
https://pytest-order.readthedocs.io/en/stable/#order-relative-to-other-tests
"""
import pytest


@pytest.mark.order(after="test_second")
def test_third():
    assert True


def test_second():
    assert True


@pytest.mark.order(before="test_second")
def test_first():
    assert True



"""Shows how to use relative sorting in test classes, see
See https://pytest-order.readthedocs.io/en/stable/#referencing-of-tests-in-other-classes-or-modules  # noqa: E501
"""
import pytest


class TestA:
    @pytest.mark.order(after="TestB::test_c")
    def test_a(self):
        assert True

    def test_b(self):
        assert True


class TestB:
    def test_c(self):
        assert True



import pytest


@pytest.fixture
def fixture_path(test_path):
    test_path.makepyfile(
        test_clss=(
            """
            import pytest

            class Test1:
                @pytest.mark.order(2)
                def test_two(self):
                    assert True

                def test_one(self):
                    assert True

            class Test2:
                def test_two(self):
                    assert True

                @pytest.mark.order(1)
                def test_one(self):
                    assert True

            @pytest.mark.order(-1)
            def test_two():
                assert True

            def test_one():
                assert True
            """
        ),
        test_fcts1=(
            """
            import pytest

            @pytest.mark.order(5)
            def test1_two():
                assert True

            def test1_one():
                assert True
            """
        ),
        test_fcts2=(
            """
            import pytest

            @pytest.mark.order(0)
            def test2_two():
                assert True

            def test2_one():
                assert True
            """
        ),
        test_fcts3=(
            """
            import pytest

            @pytest.mark.order(-2)
            def test3_two():
                assert True

            def test3_one():
                assert True
            """
        ),
        test_fcts4=(
            """
            import pytest

            def test4_one():
                assert True

            def test4_two():
                assert True
            """
        ),
    )
    yield test_path


def test_session_scope(fixture_path):
    result = fixture_path.runpytest("-v")
    result.assert_outcomes(passed=14, failed=0)
    result.stdout.fnmatch_lines(
        [
            "test_fcts2.py::test2_two PASSED",
            "test_clss.py::Test2::test_one PASSED",
            "test_clss.py::Test1::test_two PASSED",
            "test_fcts1.py::test1_two PASSED",
            "test_clss.py::Test1::test_one PASSED",
            "test_clss.py::Test2::test_two PASSED",
            "test_clss.py::test_one PASSED",
            "test_fcts1.py::test1_one PASSED",
            "test_fcts2.py::test2_one PASSED",
            "test_fcts3.py::test3_one PASSED",
            "test_fcts4.py::test4_one PASSED",
            "test_fcts4.py::test4_two PASSED",
            "test_fcts3.py::test3_two PASSED",
            "test_clss.py::test_two PASSED",
        ]
    )


def test_module_group_scope(fixture_path):
    result = fixture_path.runpytest("-v", "--order-group-scope=module")
    result.assert_outcomes(passed=14, failed=0)
    result.stdout.fnmatch_lines(
        [
            "test_fcts2.py::test2_two PASSED",
            "test_fcts2.py::test2_one PASSED",
            "test_clss.py::Test2::test_one PASSED",
            "test_clss.py::Test1::test_two PASSED",
            "test_clss.py::Test1::test_one PASSED",
            "test_clss.py::Test2::test_two PASSED",
            "test_clss.py::test_one PASSED",
            "test_clss.py::test_two PASSED",
            "test_fcts1.py::test1_two PASSED",
            "test_fcts1.py::test1_one PASSED",
            "test_fcts4.py::test4_one PASSED",
            "test_fcts4.py::test4_two PASSED",
            "test_fcts3.py::test3_one PASSED",
            "test_fcts3.py::test3_two PASSED",
        ]
    )


def test_class_group_scope(fixture_path):
    result = fixture_path.runpytest("-v", "--order-group-scope=class")
    result.assert_outcomes(passed=14, failed=0)
    result.stdout.fnmatch_lines(
        [
            "test_fcts2.py::test2_two PASSED",
            "test_fcts2.py::test2_one PASSED",
            "test_clss.py::Test2::test_one PASSED",
            "test_clss.py::Test2::test_two PASSED",
            "test_clss.py::Test1::test_two PASSED",
            "test_clss.py::Test1::test_one PASSED",
            "test_clss.py::test_one PASSED",
            "test_clss.py::test_two PASSED",
            "test_fcts1.py::test1_two PASSED",
            "test_fcts1.py::test1_one PASSED",
            "test_fcts4.py::test4_one PASSED",
            "test_fcts4.py::test4_two PASSED",
            "test_fcts3.py::test3_one PASSED",
            "test_fcts3.py::test3_two PASSED",
        ]
    )


def test_class_group_scope_module_scope(fixture_path):
    result = fixture_path.runpytest(
        "-v", "--order-group-scope=class", "--order-scope=module"
    )
    result.assert_outcomes(passed=14, failed=0)
    result.stdout.fnmatch_lines(
        [
            "test_clss.py::Test2::test_one PASSED",
            "test_clss.py::Test2::test_two PASSED",
            "test_clss.py::Test1::test_two PASSED",
            "test_clss.py::Test1::test_one PASSED",
            "test_clss.py::test_one PASSED",
            "test_clss.py::test_two PASSED",
            "test_fcts1.py::test1_two PASSED",
            "test_fcts1.py::test1_one PASSED",
            "test_fcts2.py::test2_two PASSED",
            "test_fcts2.py::test2_one PASSED",
            "test_fcts3.py::test3_one PASSED",
            "test_fcts3.py::test3_two PASSED",
            "test_fcts4.py::test4_one PASSED",
            "test_fcts4.py::test4_two PASSED",
        ]
    )


@pytest.mark.skipif(
    pytest.__version__.startswith("3.7."),
    reason="Warning does not appear in output in pytest < 3.8",
)
def test_invalid_scope(fixture_path):
    result = fixture_path.runpytest("-v", "--order-group-scope=function")
    result.assert_outcomes(passed=14, failed=0)
    result.stdout.fnmatch_lines(
        ["*UserWarning: Unknown order group scope 'function', ignoring it.*"]
    )


@pytest.mark.skipif(
    pytest.__version__.startswith("3.7."),
    reason="Warning does not appear in output in pytest < 3.8",
)
def test_ignored_scope(fixture_path):
    result = fixture_path.runpytest(
        "-v", "--order-group-scope=session", "--order-scope=module"
    )
    result.assert_outcomes(passed=14, failed=0)
    result.stdout.fnmatch_lines(
        ["*UserWarning: Group scope is larger than order scope, ignoring it."]
    )



import pytest


@pytest.fixture
def fixture_path(test_path):
    test_path.makepyfile(
        test_rel1=(
            """
            import pytest

            class Test1:
                @pytest.mark.order(after='Test2::test_two')
                def test_one(self):
                    assert True

                def test_two(self):
                    assert True

            class Test2:
                @pytest.mark.order(after='invalid')
                def test_one(self):
                    assert True

                @pytest.mark.order(after='test_rel3.py::test_two')
                def test_two(self):
                    assert True
            """
        ),
        test_rel2=(
            """
            def test_one():
                assert True

            def test_two():
                assert True
            """
        ),
        test_rel3=(
            """
            import pytest

            def test_one():
                assert True

            @pytest.mark.order(before='test_rel2.py::test_one')
            def test_two():
                assert True
            """
        ),
        test_rel4=(
            """
            import pytest

            def test_one():
                assert True

            def test_two():
                assert True
            """
        ),
    )
    yield test_path


def test_session_scope(fixture_path):
    result = fixture_path.runpytest("-v")
    result.assert_outcomes(passed=10, failed=0)
    result.stdout.fnmatch_lines(
        [
            "*'invalid' - ignoring the marker.*",
            "test_rel1.py::Test1::test_two PASSED",
            "test_rel1.py::Test2::test_one PASSED",
            "test_rel3.py::test_two PASSED",
            "test_rel1.py::Test2::test_two PASSED",
            "test_rel1.py::Test1::test_one PASSED",
            "test_rel2.py::test_one PASSED",
            "test_rel2.py::test_two PASSED",
            "test_rel3.py::test_one PASSED",
            "test_rel4.py::test_one PASSED",
            "test_rel4.py::test_two PASSED",
        ]
    )


def test_module_group_scope(fixture_path):
    result = fixture_path.runpytest("-v", "--order-group-scope=module")
    result.assert_outcomes(passed=10, failed=0)
    result.stdout.fnmatch_lines(
        [
            "test_rel3.py::test_one PASSED",
            "test_rel3.py::test_two PASSED",
            "test_rel1.py::Test1::test_two PASSED",
            "test_rel1.py::Test2::test_one PASSED",
            "test_rel1.py::Test2::test_two PASSED",
            "test_rel1.py::Test1::test_one PASSED",
            "test_rel2.py::test_one PASSED",
            "test_rel2.py::test_two PASSED",
            "test_rel4.py::test_one PASSED",
            "test_rel4.py::test_two PASSED",
        ]
    )


def test_class_group_scope(fixture_path):
    result = fixture_path.runpytest("-v", "--order-group-scope=class")
    result.assert_outcomes(passed=10, failed=0)
    result.stdout.fnmatch_lines(
        [
            "test_rel3.py::test_one PASSED",
            "test_rel3.py::test_two PASSED",
            "test_rel1.py::Test2::test_one PASSED",
            "test_rel1.py::Test2::test_two PASSED",
            "test_rel1.py::Test1::test_one PASSED",
            "test_rel1.py::Test1::test_two PASSED",
            "test_rel2.py::test_one PASSED",
            "test_rel2.py::test_two PASSED",
            "test_rel4.py::test_one PASSED",
            "test_rel4.py::test_two PASSED",
        ]
    )


def test_class_group_scope_module_scope(fixture_path):
    result = fixture_path.runpytest(
        "-v", "--order-group-scope=class", "--order-scope=module"
    )
    result.assert_outcomes(passed=10, failed=0)
    result.stdout.fnmatch_lines(
        [
            "test_rel1.py::Test2::test_one PASSED",
            "test_rel1.py::Test2::test_two PASSED",
            "test_rel1.py::Test1::test_one PASSED",
            "test_rel1.py::Test1::test_two PASSED",
            "test_rel2.py::test_one PASSED",
            "test_rel2.py::test_two PASSED",
            "test_rel3.py::test_one PASSED",
            "test_rel3.py::test_two PASSED",
            "test_rel4.py::test_one PASSED",
            "test_rel4.py::test_two PASSED",
        ]
    )


def test_rel_class_mark_with_order_mark(test_path):
    test_path.makepyfile(
        test_class_rel="""
        import pytest

        class Test1:
            def test_1(self): pass

            @pytest.mark.order(1)
            def test_2(self): pass

        @pytest.mark.order(before="Test1")
        class Test2:
            def test_1(self): pass

            @pytest.mark.order(1)
            def test_2(self): pass
        """
    )
    result = test_path.runpytest("-v", "--order-group-scope=class")
    result.assert_outcomes(passed=4, failed=0)
    result.stdout.fnmatch_lines(
        [
            "test_class_rel.py::Test2::test_2 PASSED",
            "test_class_rel.py::Test2::test_1 PASSED",
            "test_class_rel.py::Test1::test_2 PASSED",
            "test_class_rel.py::Test1::test_1 PASSED",
        ]
    )



from textwrap import dedent

import pytest
import pytest_order


def test_xdist_ordering(tmpdir):
    testname = str(tmpdir.join("first_test.py"))
    with open(testname, "w") as fi:
        fi.write(
            dedent(
                """
            import pytest

            val = 1

            @pytest.mark.order("second")
            def test_second_integer():
                global val
                assert val == 2
                val += 1

            def test_last_integer():
                assert val == 3

            @pytest.mark.order("first")
            def test_first_integer():
                global val
                assert val == 1
                val += 1
            """
            )
        )

    testname = str(tmpdir.join("second_test.py"))
    with open(testname, "w") as fi:
        fi.write(
            dedent(
                """
            import pytest

            val = "frog"

            @pytest.mark.order("second")
            def test_second_string():
                global val
                assert val == "goat"
                val = "fish"

            def test_last_string():
                assert val == "fish"

            @pytest.mark.order("first")
            def test_first_string():
                global val
                assert val == "frog"
                val = "goat"
            """
            )
        )
    # With `loadfile`, the tests should pass
    args = ["-n3", "--dist=loadfile", str(tmpdir)]
    ret = pytest.main(args, [pytest_order])
    assert ret == 0

    # Without `loadfile`, the tests should fail
    args = ["-n3", str(tmpdir)]
    ret = pytest.main(args, [pytest_order])
    assert ret == 1



import re

import pytest

import pytest_order


def test_version_exists():
    assert hasattr(pytest_order, "__version__")


def test_version_valid():
    # check for PEP 440 conform version
    assert re.match(
        r"\d+(\.\d+)*((a|b|rc)\d+)?(\.post\d)?(\.dev\d)?$",
        pytest_order.__version__,
    )


def test_markers_registered(capsys):
    pytest.main(["--markers"])
    out, err = capsys.readouterr()
    assert "@pytest.mark.order" in out
    # only order is supported as marker
    assert out.count("Provided by pytest-order.") == 1


def tests_working_without_dependency(test_path):
    """Make sure no other plugins are needed in settings."""
    test_path.makepyfile(
        test_a=(
            """
            import pytest

            def test_a(): pass

            @pytest.mark.order(0)
            def test_b(): pass
            """
        )
    )
    result = test_path.runpytest(
        "-v", "-p", "no:xdist", "-p", "no:dependency", "-p", "no:mock"
    )
    result.stdout.fnmatch_lines(
        [
            "test_a.py::test_b PASSED",
            "test_a.py::test_a PASSED",
        ]
    )



from textwrap import dedent

import pytest


@pytest.fixture
def fixture_path(test_path):
    test_a_contents = dedent(
        """
        import pytest

        @pytest.mark.order(4)
        def test_four(): pass

        @pytest.mark.order(3)
        def test_three(): pass
        """
    )
    test_b_contents = dedent(
        """
        import pytest

        @pytest.mark.order(2)
        def test_two(): pass

        @pytest.mark.order(1)
        def test_one(): pass
        """
    )
    for i in range(3):
        sub_path = "feature{}".format(i)
        test_path.mkpydir(sub_path)
        path = test_path.tmpdir.join(sub_path, "test_a.py")
        path.write(test_a_contents)
        path = test_path.tmpdir.join(sub_path, "test_b.py")
        path.write(test_b_contents)
    yield test_path


def test_session_scope(fixture_path):
    result = fixture_path.runpytest("-v")
    result.assert_outcomes(passed=12, failed=0)
    result.stdout.fnmatch_lines(
        [
            "feature0/test_b.py::test_one PASSED",
            "feature1/test_b.py::test_one PASSED",
            "feature2/test_b.py::test_one PASSED",
            "feature0/test_b.py::test_two PASSED",
            "feature1/test_b.py::test_two PASSED",
            "feature2/test_b.py::test_two PASSED",
            "feature0/test_a.py::test_three PASSED",
            "feature1/test_a.py::test_three PASSED",
            "feature2/test_a.py::test_three PASSED",
            "feature0/test_a.py::test_four PASSED",
            "feature1/test_a.py::test_four PASSED",
            "feature2/test_a.py::test_four PASSED",
        ]
    )


def test_dir_level0(fixture_path, capsys):
    """Same as session scope."""
    result = fixture_path.runpytest("-v", "--order-scope-level=0")
    result.assert_outcomes(passed=12, failed=0)
    result.stdout.fnmatch_lines(
        [
            "feature0/test_b.py::test_one PASSED",
            "feature1/test_b.py::test_one PASSED",
            "feature2/test_b.py::test_one PASSED",
            "feature0/test_b.py::test_two PASSED",
            "feature1/test_b.py::test_two PASSED",
            "feature2/test_b.py::test_two PASSED",
            "feature0/test_a.py::test_three PASSED",
            "feature1/test_a.py::test_three PASSED",
            "feature2/test_a.py::test_three PASSED",
            "feature0/test_a.py::test_four PASSED",
            "feature1/test_a.py::test_four PASSED",
            "feature2/test_a.py::test_four PASSED",
        ]
    )


def test_dir_level1(fixture_path, capsys):
    """Orders per feature path."""
    result = fixture_path.runpytest("-v", "--order-scope-level=1")
    result.assert_outcomes(passed=12, failed=0)
    result.stdout.fnmatch_lines(
        [
            "feature0/test_b.py::test_one PASSED",
            "feature0/test_b.py::test_two PASSED",
            "feature0/test_a.py::test_three PASSED",
            "feature0/test_a.py::test_four PASSED",
            "feature1/test_b.py::test_one PASSED",
            "feature1/test_b.py::test_two PASSED",
            "feature1/test_a.py::test_three PASSED",
            "feature1/test_a.py::test_four PASSED",
            "feature2/test_b.py::test_one PASSED",
            "feature2/test_b.py::test_two PASSED",
            "feature2/test_a.py::test_three PASSED",
            "feature2/test_a.py::test_four PASSED",
        ]
    )


def test_dir_level2(fixture_path, capsys):
    """Same as module scope."""
    result = fixture_path.runpytest("-v", "--order-scope-level=2")
    result.assert_outcomes(passed=12, failed=0)
    result.stdout.fnmatch_lines(
        [
            "feature0/test_a.py::test_three PASSED",
            "feature0/test_a.py::test_four PASSED",
            "feature0/test_b.py::test_one PASSED",
            "feature0/test_b.py::test_two PASSED",
            "feature1/test_a.py::test_three PASSED",
            "feature1/test_a.py::test_four PASSED",
            "feature1/test_b.py::test_one PASSED",
            "feature1/test_b.py::test_two PASSED",
            "feature2/test_a.py::test_three PASSED",
            "feature2/test_a.py::test_four PASSED",
            "feature2/test_b.py::test_one PASSED",
            "feature2/test_b.py::test_two PASSED",
        ]
    )


@pytest.mark.skipif(
    pytest.__version__.startswith("3.7."),
    reason="Warning does not appear in output in pytest < 3.8",
)
def test_invalid_scope(fixture_path):
    result = fixture_path.runpytest(
        "-v", "--order-scope=module", "--order-scope-level=1"
    )
    result.assert_outcomes(passed=12, failed=0)
    result.stdout.fnmatch_lines(
        [
            "*UserWarning: order-scope-level cannot be used "
            "together with --order-scope=module*"
        ]
    )



"""Test showing that sorting is not done if it would break an existing
dependency.
See https://pytest-order.readthedocs.io/en/stable/#order-dependencies
"""
import pytest


def test_a():
    assert True


@pytest.mark.dependency(depends=["test_a"])
@pytest.mark.order("first")
def test_b():
    assert True



import pytest


def test_ignore_order_with_dependency(item_names_for):
    tests_content = """
        import pytest

        @pytest.mark.dependency()
        def test_a():
            pass

        @pytest.mark.dependency(depends=["test_a"])
        @pytest.mark.order("first")
        def test_b():
            pass
        """
    assert item_names_for(tests_content) == ["test_a", "test_b"]


def test_order_with_dependency(item_names_for):
    tests_content = """
        import pytest

        @pytest.mark.dependency(depends=["test_b"])
        @pytest.mark.order("second")
        def test_a():
            pass

        @pytest.mark.dependency()
        def test_b():
            pass
        """
    assert item_names_for(tests_content) == ["test_b", "test_a"]


@pytest.fixture(scope="module")
def ordered_test():
    yield (
        """
        import pytest

        @pytest.mark.dependency()
        def test_a():
            pass

        @pytest.mark.dependency(depends=["test_a"])
        def test_b():
            pass
        """
    )


def test_dependency_already_ordered_default(ordered_test, item_names_for):
    assert item_names_for(ordered_test) == ["test_a", "test_b"]


def test_dependency_already_ordered_with_ordering(
    ordered_test, item_names_for, order_dependencies
):
    assert item_names_for(ordered_test) == ["test_a", "test_b"]


@pytest.fixture(scope="module")
def order_dependency_test():
    yield (
        """
        import pytest

        @pytest.mark.dependency(depends=["test_b"])
        def test_a():
            pass

        @pytest.mark.dependency()
        def test_b():
            pass
        """
    )


def test_order_dependency_default(order_dependency_test, item_names_for):
    assert item_names_for(order_dependency_test) == ["test_a", "test_b"]


def test_order_dependency_ordered(
    order_dependency_test, item_names_for, order_dependencies
):
    assert item_names_for(order_dependency_test) == ["test_b", "test_a"]


@pytest.fixture(scope="module")
def multiple_dependencies_test():
    yield (
        """
        import pytest

        @pytest.mark.dependency(depends=["test_b", "test_c"])
        def test_a():
            pass

        @pytest.mark.dependency()
        def test_b():
            pass

        @pytest.mark.dependency()
        def test_c():
            pass
        """
    )


def test_order_multiple_dependencies_default(
    multiple_dependencies_test, item_names_for
):
    assert item_names_for(multiple_dependencies_test) == ["test_a", "test_b", "test_c"]


def test_order_multiple_dependencies_ordered(
    multiple_dependencies_test, item_names_for, order_dependencies
):
    assert item_names_for(multiple_dependencies_test) == ["test_b", "test_c", "test_a"]


@pytest.fixture
def no_dep_marks(test_path):
    test_path.makepyfile(
        test_auto=(
            """
            import pytest

            @pytest.mark.dependency(depends=["test_b", "test_c"])
            def test_a():
                pass

            def test_b():
                pass

            def test_c():
                pass
            """
        )
    )
    yield test_path


def test_order_dependencies_no_auto_mark(no_dep_marks):
    no_dep_marks.makefile(
        ".ini",
        pytest=(
            """
            [pytest]
            automark_dependency = 0
            console_output_style = classic
            """
        ),
    )
    result = no_dep_marks.runpytest("-v", "--order-dependencies")
    result.assert_outcomes(passed=2, skipped=1)
    result.stdout.fnmatch_lines(
        [
            "test_auto.py::test_a SKIPPED*",
            "test_auto.py::test_b PASSED",
            "test_auto.py::test_c PASSED",
        ]
    )


def test_order_dependencies_auto_mark(no_dep_marks):
    no_dep_marks.makefile(
        ".ini",
        pytest=(
            """
            [pytest]
            automark_dependency = 1
            console_output_style = classic
            """
        ),
    )
    result = no_dep_marks.runpytest("-v", "--order-dependencies")
    result.assert_outcomes(passed=3, failed=0)
    result.stdout.fnmatch_lines(
        [
            "test_auto.py::test_b PASSED",
            "test_auto.py::test_c PASSED",
            "test_auto.py::test_a PASSED",
        ]
    )


@pytest.fixture(scope="module")
def named_dependency_test():
    yield (
        """
        import pytest

        @pytest.mark.dependency(depends=["my_test"])
        def test_a():
            pass

        @pytest.mark.dependency(name="my_test")
        def test_b():
            pass

        def test_c():
            pass
        """
    )


def test_order_named_dependency_default(named_dependency_test, item_names_for):
    assert item_names_for(named_dependency_test) == ["test_a", "test_b", "test_c"]


def test_order_named_dependency_ordered(
    named_dependency_test, item_names_for, order_dependencies
):
    assert item_names_for(named_dependency_test) == ["test_b", "test_a", "test_c"]


def test_dependency_in_class(item_names_for, order_dependencies):
    tests_content = """
        import pytest

        class Test:
            @pytest.mark.dependency(depends=["Test::test_c"])
            def test_a(self):
                assert True

            @pytest.mark.dependency(depends=["Test::test_c"])
            def test_b(self):
                assert True

            @pytest.mark.dependency()
            def test_c(self):
                assert True
        """
    assert item_names_for(tests_content) == [
        "Test::test_c",
        "Test::test_a",
        "Test::test_b",
    ]


def test_unresolved_dependency_in_class(item_names_for, order_dependencies, capsys):
    tests_content = """
        import pytest

        class Test:
            @pytest.mark.dependency(depends=["test_c"])
            def test_a(self):
                assert True

            @pytest.mark.dependency(depends=["test_c"])
            def test_b(self):
                assert True

            @pytest.mark.dependency()
            def test_c(self):
                assert True
        """
    assert item_names_for(tests_content) == [
        "Test::test_a",
        "Test::test_b",
        "Test::test_c",
    ]
    out, err = capsys.readouterr()
    warning = "Cannot resolve the dependency marker 'test_c' - ignoring it"
    assert warning in out


def test_named_dependency_in_class(item_names_for, order_dependencies):
    tests_content = """
        import pytest

        class Test:
            @pytest.mark.dependency(name="test_1", depends=["test_3"])
            def test_a(self):
                assert True

            @pytest.mark.dependency(name="test_2", depends=["test_3"])
            def test_b(self):
                assert True

            @pytest.mark.dependency(name="test_3")
            def test_c(self):
                assert True
        """
    assert item_names_for(tests_content) == [
        "Test::test_c",
        "Test::test_a",
        "Test::test_b",
    ]


def test_dependencies_in_classes(item_names_for, order_dependencies):
    tests_content = """
        import pytest

        class TestA:
            @pytest.mark.dependency(depends=["test_2"])
            def test_a(self):
                assert True

            @pytest.mark.dependency(depends=["TestB::test_b"])
            def test_b(self):
                assert True

            def test_c(self):
                assert True

        class TestB:
            @pytest.mark.dependency(name="test_2")
            def test_a(self):
                assert True

            @pytest.mark.dependency()
            def test_b(self):
                assert True

            def test_c(self):
                assert True
        """
    assert item_names_for(tests_content) == [
        "TestA::test_c",
        "TestB::test_a",
        "TestA::test_a",
        "TestB::test_b",
        "TestA::test_b",
        "TestB::test_c",
    ]


def test_class_scope_dependencies(item_names_for, order_dependencies):
    tests_content = """
        import pytest

        class TestA:
            @pytest.mark.dependency(depends=["test_c"], scope='class')
            def test_a(self):
                assert True

            def test_b(self):
                assert True

            @pytest.mark.dependency
            def test_c(self):
                assert True
        """
    assert item_names_for(tests_content) == [
        "TestA::test_b",
        "TestA::test_c",
        "TestA::test_a",
    ]


@pytest.mark.skipif(
    pytest.__version__.startswith("3.7."),
    reason="pytest-dependency < 0.5 does not support session scope",
)
def test_named_dependency_in_modules(test_path):
    test_path.makepyfile(
        test_ndep1=(
            """
            import pytest

            class Test1:
                def test_one(self):
                    assert True

                @pytest.mark.dependency(
                    depends=['dep2_test_one'], scope='session'
                )
                def test_two(self):
                    assert True
            """
        ),
        test_ndep2=(
            """
            import pytest

            @pytest.mark.dependency(name='dep2_test_one')
            def test_one():
                assert True

            def test_two():
                assert True
            """
        ),
    )

    result = test_path.runpytest("-v", "--order-dependencies")
    result.assert_outcomes(passed=4, failed=0)
    result.stdout.fnmatch_lines(
        [
            "test_ndep1.py::Test1::test_one PASSED",
            "test_ndep2.py::test_one PASSED",
            "test_ndep1.py::Test1::test_two PASSED",
            "test_ndep2.py::test_two PASSED",
        ]
    )


@pytest.mark.skipif(
    pytest.__version__.startswith("3.7."),
    reason="pytest-dependency < 0.5 does not support session scope",
)
def test_dependency_in_modules(test_path):
    test_path.makepyfile(
        test_unnamed_dep1=(
            """
            import pytest

            class Test1:
                def test_one(self):
                    assert True

                @pytest.mark.dependency(
                    depends=['test_unnamed_dep2.py::test_one'],
                    scope='session',
                )
                def test_two(self):
                    assert True
            """
        ),
        test_unnamed_dep2=(
            """
            import pytest

            @pytest.mark.dependency
            def test_one():
                assert True

            def test_two():
                assert True
            """
        ),
    )

    result = test_path.runpytest("-v", "--order-dependencies")
    result.assert_outcomes(passed=4, failed=0)
    result.stdout.fnmatch_lines(
        [
            "test_unnamed_dep1.py::Test1::test_one PASSED",
            "test_unnamed_dep2.py::test_one PASSED",
            "test_unnamed_dep1.py::Test1::test_two PASSED",
            "test_unnamed_dep2.py::test_two PASSED",
        ]
    )


def test_same_dependency_in_modules(test_path):
    # regression test - make sure that the same dependency in different
    # modules works correctly
    test_path.makepyfile(
        test_module_dep1=(
            """
            import pytest

            @pytest.mark.dependency(depends=['test_two'])
            def test_one():
                assert True

            @pytest.mark.dependency
            def test_two():
                assert True
            """
        ),
        test_module_dep2=(
            """
            import pytest

            @pytest.mark.dependency(depends=['test_two'])
            def test_one():
                assert True

            @pytest.mark.dependency
            def test_two():
                assert True
            """
        ),
    )
    result = test_path.runpytest("-v", "--order-dependencies")
    result.assert_outcomes(passed=4, failed=0)
    result.stdout.fnmatch_lines(
        [
            "test_module_dep1.py::test_two PASSED",
            "test_module_dep1.py::test_one PASSED",
            "test_module_dep2.py::test_two PASSED",
            "test_module_dep2.py::test_one PASSED",
        ]
    )


def test_unknown_dependency(item_names_for, order_dependencies, capsys):
    tests_content = """
        import pytest

        class Test:
            def test_a(self):
                assert True

            @pytest.mark.dependency(depends=["test_3"])
            def test_b(self):
                assert True

            def test_c(self):
                assert True
        """
    assert item_names_for(tests_content) == [
        "Test::test_a",
        "Test::test_b",
        "Test::test_c",
    ]
    out, err = capsys.readouterr()
    warning = "Cannot resolve the dependency marker 'test_3' - ignoring it."
    assert warning in out


def test_unsupported_order_with_dependency(item_names_for):
    test_content = """
        import pytest

        @pytest.mark.dependency(depends=["test_2"])
        @pytest.mark.order("unknown")
        def test_1():
            pass

        def test_2():
            pass
        """
    with pytest.warns(UserWarning, match="Unknown order attribute:'unknown'"):
        assert item_names_for(test_content) == ["test_1", "test_2"]



"""Test showing the behavior of the --order-dependencies option.
See https://pytest-order.readthedocs.io/en/stable/#order-dependencies
"""
import pytest


@pytest.mark.dependency(depends=["test_b"])
def test_a():
    assert True


@pytest.mark.dependency
def test_b():
    assert True



from unittest import mock
from textwrap import dedent

import pytest
from perf_tests.util import TimedSorter

pytest_plugins = ["pytester"]


@pytest.fixture
def fixture_path_ordinal(testdir):
    for i_mod in range(10):
        test_name = testdir.tmpdir.join("test_performance{}.py".format(i_mod))
        test_contents = "import pytest\n"
        for i in range(100):
            test_contents += dedent(
                """
                @pytest.mark.order({})
                def test_{}():
                    assert True
                """
            ).format(50 - i, i)
        test_name.write(test_contents)
    yield testdir


@mock.patch("pytest_order.plugin.Sorter", TimedSorter)
def test_performance_ordinal(fixture_path_ordinal):
    TimedSorter.nr_marks = 1000
    fixture_path_ordinal.runpytest("--quiet")
    assert TimedSorter.elapsed < 0.02



import pytest


@pytest.fixture
def fixture_path(test_path):
    test_path.makepyfile(
        test_named_dep1=(
            """
            import pytest

            class Test1:
                @pytest.mark.dependency(depends=['Test2_test2'])
                def test_one(self):
                    assert True

                def test_two(self):
                    assert True

            class Test2:
                def test_one(self):
                    assert True

                @pytest.mark.dependency(
                    name='Test2_test2',
                    depends=['dep3_test_two'],
                    scope='session',
                )
                def test_two(self):
                    assert True
            """
        ),
        test_named_dep2=(
            """
            import pytest

            @pytest.mark.dependency(
                depends=['dep3_test_two'], scope='session'
            )
            def test_one():
                assert True

            def test_two():
                assert True
            """
        ),
        test_named_dep3=(
            """
            import pytest

            def test_one():
                assert True

            @pytest.mark.dependency(name='dep3_test_two')
            def test_two():
                assert True
            """
        ),
        test_named_dep4=(
            """
            import pytest

            def test_one():
                assert True

            def test_two():
                assert True
            """
        ),
    )
    yield test_path


@pytest.mark.skipif(
    pytest.__version__.startswith("3.7."),
    reason="pytest-dependency < 0.5 does not support session scope",
)
def test_session_scope(fixture_path):
    result = fixture_path.runpytest("-v", "--order-dependencies")
    result.assert_outcomes(passed=10, failed=0)
    result.stdout.fnmatch_lines(
        [
            "test_named_dep1.py::Test1::test_two PASSED",
            "test_named_dep1.py::Test2::test_one PASSED",
            "test_named_dep2.py::test_two PASSED",
            "test_named_dep3.py::test_one PASSED",
            "test_named_dep3.py::test_two PASSED",
            "test_named_dep1.py::Test2::test_two PASSED",
            "test_named_dep1.py::Test1::test_one PASSED",
            "test_named_dep2.py::test_one PASSED",
            "test_named_dep4.py::test_one PASSED",
            "test_named_dep4.py::test_two PASSED",
        ]
    )


@pytest.mark.skipif(
    pytest.__version__.startswith("3.7."),
    reason="pytest-dependency < 0.5 does not support session scope",
)
def test_module_group_scope(fixture_path):
    result = fixture_path.runpytest(
        "-v", "--order-dependencies", "--order-group-scope=module"
    )
    result.assert_outcomes(passed=10, failed=0)
    result.stdout.fnmatch_lines(
        [
            "test_named_dep3.py::test_one PASSED",
            "test_named_dep3.py::test_two PASSED",
            "test_named_dep1.py::Test1::test_two PASSED",
            "test_named_dep1.py::Test2::test_one PASSED",
            "test_named_dep1.py::Test2::test_two PASSED",
            "test_named_dep1.py::Test1::test_one PASSED",
            "test_named_dep2.py::test_one PASSED",
            "test_named_dep2.py::test_two PASSED",
            "test_named_dep4.py::test_one PASSED",
            "test_named_dep4.py::test_two PASSED",
        ]
    )


@pytest.mark.skipif(
    pytest.__version__.startswith("3.7."),
    reason="pytest-dependency < 0.5 does not support session scope",
)
def test_class_group_scope(fixture_path):
    result = fixture_path.runpytest(
        "-v", "--order-dependencies", "--order-group-scope=class"
    )
    result.assert_outcomes(passed=10, failed=0)
    result.stdout.fnmatch_lines(
        [
            "test_named_dep3.py::test_one PASSED",
            "test_named_dep3.py::test_two PASSED",
            "test_named_dep1.py::Test2::test_one PASSED",
            "test_named_dep1.py::Test2::test_two PASSED",
            "test_named_dep1.py::Test1::test_one PASSED",
            "test_named_dep1.py::Test1::test_two PASSED",
            "test_named_dep2.py::test_one PASSED",
            "test_named_dep2.py::test_two PASSED",
            "test_named_dep4.py::test_one PASSED",
            "test_named_dep4.py::test_two PASSED",
        ]
    )


@pytest.mark.skipif(
    pytest.__version__.startswith("3.7."),
    reason="pytest-dependency < 0.5 does not support session scope",
)
def test_class_group_scope_module_scope(fixture_path):
    result = fixture_path.runpytest(
        "-v",
        "--order-dependencies",
        "--order-group-scope=class",
        "--order-scope=module",
    )
    result.assert_outcomes(passed=7, skipped=3)
    result.stdout.fnmatch_lines(
        [
            "test_named_dep1.py::Test2::test_one PASSED",
            "test_named_dep1.py::Test2::test_two SKIPPED*",
            "test_named_dep1.py::Test1::test_one SKIPPED*",
            "test_named_dep1.py::Test1::test_two PASSED",
            "test_named_dep2.py::test_one SKIPPED*",
            "test_named_dep2.py::test_two PASSED",
            "test_named_dep3.py::test_one PASSED",
            "test_named_dep3.py::test_two PASSED",
            "test_named_dep4.py::test_one PASSED",
            "test_named_dep4.py::test_two PASSED",
        ]
    )



"""Shows how to use ordinal names for sorting.
See https://pytest-order.readthedocs.io/en/stable/#order-using-ordinals
"""
import pytest


@pytest.mark.order("second_to_last")
def test_three():
    assert True


@pytest.mark.order("last")
def test_four():
    assert True


@pytest.mark.order("second")
def test_two():
    assert True


@pytest.mark.order("first")
def test_one():
    assert True



"""One of the test files showing the use of the --order-scope option.
See https://pytest-order.readthedocs.io/en/stable/#order-scope
"""
import pytest


@pytest.mark.order(2)
def test2():
    pass


@pytest.mark.order(1)
def test1():
    pass



"""Shows how combined ordinal and relative sorting is handled.
See https://pytest-order.readthedocs.io/en/stable/#combination-of-absolute-and-relative-ordering  # noqa: E501
"""
import pytest


@pytest.mark.order(index=0, after="test_second")
def test_first():
    assert True


@pytest.mark.order(1)
def test_second():
    assert True



import pytest


@pytest.fixture
def fixture_path(test_path):
    test_path.makepyfile(
        test_classes=(
            """
            import pytest

            class Test1:
                @pytest.mark.order("last")
                def test_two(self):
                    assert True

                @pytest.mark.order("first")
                def test_one(self):
                    assert True

            class Test2:
                @pytest.mark.order("last")
                def test_two(self):
                    assert True

                @pytest.mark.order("first")
                def test_one(self):
                    assert True

            @pytest.mark.order("last")
            def test_two():
                assert True

            @pytest.mark.order("first")
            def test_one():
                assert True
            """
        ),
        test_functions1=(
            """
            import pytest

            @pytest.mark.order("last")
            def test1_two():
                assert True

            @pytest.mark.order("first")
            def test1_one():
                assert True
            """
        ),
        test_functions2=(
            """
            import pytest

            @pytest.mark.order("last")
            def test2_two():
                assert True

            @pytest.mark.order("first")
            def test2_one():
                assert True
            """
        ),
    )
    yield test_path


def test_session_scope(fixture_path):
    result = fixture_path.runpytest("-v")
    result.assert_outcomes(passed=10, failed=0)
    result.stdout.fnmatch_lines(
        [
            "test_classes.py::Test1::test_one PASSED",
            "test_classes.py::Test2::test_one PASSED",
            "test_classes.py::test_one PASSED",
            "test_functions1.py::test1_one PASSED",
            "test_functions2.py::test2_one PASSED",
            "test_classes.py::Test1::test_two PASSED",
            "test_classes.py::Test2::test_two PASSED",
            "test_classes.py::test_two PASSED",
            "test_functions1.py::test1_two PASSED",
            "test_functions2.py::test2_two PASSED",
        ]
    )


def test_module_scope(fixture_path):
    result = fixture_path.runpytest("-v", "--order-scope=module")
    result.assert_outcomes(passed=10, failed=0)
    result.stdout.fnmatch_lines(
        [
            "test_classes.py::Test1::test_one PASSED",
            "test_classes.py::Test2::test_one PASSED",
            "test_classes.py::test_one PASSED",
            "test_classes.py::Test1::test_two PASSED",
            "test_classes.py::Test2::test_two PASSED",
            "test_classes.py::test_two PASSED",
            "test_functions1.py::test1_one PASSED",
            "test_functions1.py::test1_two PASSED",
            "test_functions2.py::test2_one PASSED",
            "test_functions2.py::test2_two PASSED",
        ]
    )


def test_class_scope(fixture_path):
    result = fixture_path.runpytest("-v", "--order-scope=class")
    result.assert_outcomes(passed=10, failed=0)
    result.stdout.fnmatch_lines(
        [
            "test_classes.py::Test1::test_one PASSED",
            "test_classes.py::Test1::test_two PASSED",
            "test_classes.py::Test2::test_one PASSED",
            "test_classes.py::Test2::test_two PASSED",
            "test_classes.py::test_one PASSED",
            "test_classes.py::test_two PASSED",
            "test_functions1.py::test1_one PASSED",
            "test_functions1.py::test1_two PASSED",
            "test_functions2.py::test2_one PASSED",
            "test_functions2.py::test2_two PASSED",
        ]
    )


@pytest.mark.skipif(
    pytest.__version__.startswith("3.7."),
    reason="Warning does not appear in output in pytest < 3.8",
)
def test_invalid_scope(fixture_path):
    result = fixture_path.runpytest("-v", "--order-scope=function")
    result.assert_outcomes(passed=10, failed=0)
    result.stdout.fnmatch_lines(
        ["*UserWarning: Unknown order scope 'function', ignoring it.*"]
    )



def test_ordinal_class_marks(item_names_for):
    tests_content = """
        import pytest

        @pytest.mark.order(1)
        class Test1:
            def test_1(self): pass
            def test_2(self): pass

        @pytest.mark.order(0)
        class Test2:
            def test_1(self): pass
            def test_2(self): pass

        """
    assert item_names_for(tests_content) == [
        "Test2::test_1",
        "Test2::test_2",
        "Test1::test_1",
        "Test1::test_2",
    ]


def test_after_class_mark(item_names_for):
    tests_content = """
        import pytest

        @pytest.mark.order(after="Test2")
        class Test1:
            def test_1(self): pass
            def test_2(self): pass

        class Test2:
            def test_1(self): pass
            def test_2(self): pass
        """
    assert item_names_for(tests_content) == [
        "Test2::test_1",
        "Test2::test_2",
        "Test1::test_1",
        "Test1::test_2",
    ]


def test_invalid_class_mark(item_names_for, capsys):
    tests_content = """
        import pytest

        @pytest.mark.order(after="Test3")
        class Test1:
            def test_1(self): pass
            def test_2(self): pass

        class Test2:
            def test_1(self): pass
            def test_2(self): pass
        """
    assert item_names_for(tests_content) == [
        "Test1::test_1",
        "Test1::test_2",
        "Test2::test_1",
        "Test2::test_2",
    ]
    out, err = capsys.readouterr()
    assert (
        "WARNING: cannot execute 'test_2' relative to others: "
        "'Test3' - ignoring the marker" in out
    )


def test_before_class_mark(item_names_for):
    tests_content = """
        import pytest

        class Test1:
            def test_1(self): pass
            def test_2(self): pass

        @pytest.mark.order(before="Test1")
        class Test2:
            def test_1(self): pass
            def test_2(self): pass
        """
    assert item_names_for(tests_content) == [
        "Test2::test_1",
        "Test2::test_2",
        "Test1::test_1",
        "Test1::test_2",
    ]


def test_after_class_marks_for_single_test_in_class(item_names_for):
    tests_content = """
        import pytest

        @pytest.mark.order(after="Test2::test_1")
        class Test1:
            def test_1(self): pass
            def test_2(self): pass

        class Test2:
            def test_1(self): pass
            def test_2(self): pass
        """
    assert item_names_for(tests_content) == [
        "Test2::test_1",
        "Test1::test_1",
        "Test1::test_2",
        "Test2::test_2",
    ]


def test_before_class_marks_for_single_test_in_class(item_names_for):
    tests_content = """
        import pytest

        class Test1:
            def test_1(self): pass
            def test_2(self): pass

        @pytest.mark.order(before="Test1::test_2")
        class Test2:
            def test_1(self): pass
            def test_2(self): pass
        """
    assert item_names_for(tests_content) == [
        "Test1::test_1",
        "Test2::test_1",
        "Test2::test_2",
        "Test1::test_2",
    ]


def test_after_class_marks_for_single_test(item_names_for):
    tests_content = """
        import pytest

        @pytest.mark.order(after="test_1")
        class Test1:
            def test_1(self): pass
            def test_2(self): pass

        def test_1(): pass

        class Test2:
            def test_1(self): pass
            def test_2(self): pass
        """
    assert item_names_for(tests_content) == [
        "test_1",
        "Test1::test_1",
        "Test1::test_2",
        "Test2::test_1",
        "Test2::test_2",
    ]


def test_before_class_marks_for_single_test(item_names_for):
    tests_content = """
        import pytest

        def test_1(): pass

        class Test1:
            def test_1(self): pass
            def test_2(self): pass

        @pytest.mark.order(before="test_1")
        class Test2:
            def test_1(self): pass
            def test_2(self): pass
        """
    assert item_names_for(tests_content) == [
        "Test2::test_1",
        "Test2::test_2",
        "test_1",
        "Test1::test_1",
        "Test1::test_2",
    ]


def test_rel_class_mark_with_order_mark(item_names_for):
    tests_content = """
        import pytest

        class Test1:
            def test_1(self): pass

            def test_2(self): pass

        @pytest.mark.order(before="Test1")
        class Test2:
            @pytest.mark.order(2)
            def test_1(self): pass

            @pytest.mark.order(1)
            def test_2(self): pass
        """
    assert item_names_for(tests_content) == [
        "Test2::test_2",
        "Test2::test_1",
        "Test1::test_1",
        "Test1::test_2",
    ]



def test_run_marker_registered(test_path):
    test_path.makepyfile(
        test_failing=(
            """
            import pytest

            @pytest.mark.order("second")
            def test_me_second():
                assert True

            def test_that_fails():
                assert False

            @pytest.mark.order("first")
            def test_me_first():
                assert True
            """
        )
    )
    result = test_path.runpytest("-v")
    result.assert_outcomes(passed=2, failed=1)
    result.stdout.fnmatch_lines(
        [
            "test_failing.py::test_me_first PASSED",
            "test_failing.py::test_me_second PASSED",
            "test_failing.py::test_that_fails FAILED",
        ]
    )

    result = test_path.runpytest("-v", "--ff", "--indulgent-ordering")
    result.assert_outcomes(passed=2, failed=1)
    result.stdout.fnmatch_lines(
        [
            "test_failing.py::test_that_fails FAILED",
            "test_failing.py::test_me_first PASSED",
            "test_failing.py::test_me_second PASSED",
        ]
    )



import pytest


def test_no_marks(item_names_for):
    tests_content = """
        def test_1(): pass

        def test_2(): pass
        """
    assert item_names_for(tests_content) == ["test_1", "test_2"]


def test_first_mark(item_names_for):
    tests_content = """
        import pytest

        def test_1(): pass

        @pytest.mark.order("first")
        def test_2(): pass
        """
    assert item_names_for(tests_content) == ["test_2", "test_1"]


def test_last_mark(item_names_for):
    tests_content = """
        import pytest

        @pytest.mark.order("last")
        def test_1(): pass

        def test_2(): pass
        """
    assert item_names_for(tests_content) == ["test_2", "test_1"]


def test_first_last_marks(item_names_for):
    tests_content = """
        import pytest

        @pytest.mark.order("last")
        def test_1(): pass

        @pytest.mark.order("first")
        def test_2(): pass

        def test_3(): pass
        """
    assert item_names_for(tests_content) == ["test_2", "test_3", "test_1"]


def test_order_marks(item_names_for):
    tests_content = """
        import pytest

        @pytest.mark.order(-1)
        def test_1(): pass

        @pytest.mark.order(-2)
        def test_2(): pass

        @pytest.mark.order(1)
        def test_3(): pass
        """
    assert item_names_for(tests_content) == ["test_3", "test_2", "test_1"]


def test_order_marks_by_index(item_names_for):
    tests_content = """
        import pytest

        @pytest.mark.order(index=-1)
        def test_1(): pass

        @pytest.mark.order(index=-2)
        def test_2(): pass

        @pytest.mark.order(index=1)
        def test_3(): pass
        """
    assert item_names_for(tests_content) == ["test_3", "test_2", "test_1"]


def test_non_contiguous_positive(item_names_for):
    tests_content = """
        import pytest

        @pytest.mark.order(10)
        def test_1(): pass

        @pytest.mark.order(20)
        def test_2(): pass

        @pytest.mark.order(5)
        def test_3(): pass
        """
    assert item_names_for(tests_content) == ["test_3", "test_1", "test_2"]


def test_non_contiguous_negative(item_names_for):
    tests_content = """
        import pytest

        @pytest.mark.order(-10)
        def test_1(): pass

        @pytest.mark.order(-20)
        def test_2(): pass

        @pytest.mark.order(-5)
        def test_3(): pass
        """
    assert item_names_for(tests_content) == ["test_2", "test_1", "test_3"]


def test_non_contiguous_inc_zero(item_names_for):
    tests_content = """
        import pytest

        @pytest.mark.order(10)
        def test_1(): pass

        @pytest.mark.order(20)
        def test_2(): pass

        @pytest.mark.order(5)
        def test_3(): pass

        @pytest.mark.order(-10)
        def test_4(): pass

        @pytest.mark.order(-20)
        def test_5(): pass

        @pytest.mark.order(-5)
        def test_6(): pass

        @pytest.mark.order(0)
        def test_7(): pass
        """
    assert item_names_for(tests_content) == [
        "test_7",
        "test_3",
        "test_1",
        "test_2",
        "test_5",
        "test_4",
        "test_6",
    ]


def test_non_contiguous_inc_none(item_names_for):
    tests_content = """
        import pytest

        @pytest.mark.order(5)
        def test_1(): pass

        @pytest.mark.order(0)
        def test_2(): pass

        @pytest.mark.order(1)
        def test_3(): pass

        @pytest.mark.order(-1)
        def test_4(): pass

        @pytest.mark.order(-5)
        def test_5(): pass

        def test_6(): pass
        """
    assert item_names_for(tests_content) == [
        "test_2",
        "test_3",
        "test_1",
        "test_6",
        "test_5",
        "test_4",
    ]


def test_first_mark_class(item_names_for):
    tests_content = """
        import pytest

        def test_1(): pass


        @pytest.mark.order("first")
        class TestSuite:

            def test_3(self): pass

            def test_2(self): pass

        """
    assert item_names_for(tests_content) == [
        "TestSuite::test_3",
        "TestSuite::test_2",
        "test_1",
    ]


def test_last_mark_class(item_names_for):
    tests_content = """
        import pytest

        @pytest.mark.order("last")
        class TestSuite:

            def test_1(self): pass

            def test_2(self): pass


        def test_3(): pass
        """
    assert item_names_for(tests_content) == [
        "test_3",
        "TestSuite::test_1",
        "TestSuite::test_2",
    ]


def test_first_last_mark_class(item_names_for):
    tests_content = """
        import pytest

        @pytest.mark.order("last")
        class TestLast:

            def test_1(self): pass

            def test_2(self): pass


        def test_0(): pass


        @pytest.mark.order("first")
        class TestFirst:

            def test_1(self): pass

            def test_2(self): pass

        """
    assert item_names_for(tests_content) == [
        "TestFirst::test_1",
        "TestFirst::test_2",
        "test_0",
        "TestLast::test_1",
        "TestLast::test_2",
    ]


def test_order_mark_class(item_names_for):
    tests_content = """
        import pytest

        @pytest.mark.order(-1)
        class TestLast:
            def test_1(self): pass
            def test_2(self): pass

        @pytest.mark.order(0)
        def test_0(): pass

        @pytest.mark.order(-2)
        class TestFirst:
            def test_1(self): pass
            def test_2(self): pass
        """
    assert item_names_for(tests_content) == [
        "test_0",
        "TestFirst::test_1",
        "TestFirst::test_2",
        "TestLast::test_1",
        "TestLast::test_2",
    ]


def test_run_ordinals(item_names_for):
    test_content = """
        import pytest

        @pytest.mark.order("second_to_last")
        def test_three():
            pass

        @pytest.mark.order("last")
        def test_four():
            pass

        @pytest.mark.order("second")
        def test_two():
            pass

        @pytest.mark.order("first")
        def test_one():
            pass
        """
    assert item_names_for(test_content) == [
        "test_one",
        "test_two",
        "test_three",
        "test_four",
    ]


def test_sparse_numbers(item_names_for):
    test_content = """
        import pytest

        @pytest.mark.order(4)
        def test_two():
            assert True

        def test_three():
            assert True

        @pytest.mark.order(2)
        def test_one():
            assert True
        """
    assert item_names_for(test_content) == ["test_one", "test_two", "test_three"]


def test_quickstart(item_names_for):
    test_content = """
        import pytest

        def test_foo():
            pass

        def test_bar():
            pass
        """
    assert item_names_for(test_content) == ["test_foo", "test_bar"]


def test_quickstart2(item_names_for):
    test_content = """
        import pytest

        @pytest.mark.order(2)
        def test_foo():
            pass

        @pytest.mark.order(1)
        def test_bar():
            pass
        """
    assert item_names_for(test_content) == ["test_bar", "test_foo"]


def test_unsupported_order(item_names_for):
    test_content = """
        import pytest

        @pytest.mark.order("unknown")
        def test_1():
            pass

        def test_2():
            pass
        """
    with pytest.warns(UserWarning, match="Unknown order attribute:'unknown'"):
        assert item_names_for(test_content) == ["test_1", "test_2"]



"""Test file showing the behavior of the --sparse-ordering option.
See https://pytest-order.readthedocs.io/en/stable/#sparse-ordering
"""
import pytest


@pytest.mark.order(3)
def test_two():
    assert True


def test_three():
    assert True


def test_four():
    assert True


@pytest.mark.order(1)
def test_one():
    assert True



#!/usr/bin/env python

from setuptools import setup
import os

__here__ = os.path.abspath(os.path.dirname(__file__))

from pytest_order import __version__

with open(os.path.join(__here__, "README.md")) as f:
    LONG_DESCRIPTION = f.read()


setup(
    name="pytest-order",
    description="pytest plugin to run your tests in a specific order",
    long_description=LONG_DESCRIPTION,
    long_description_content_type="text/markdown",
    version=__version__,
    author="mrbean-bremen",
    author_email="hansemrbean@googlemail.com",
    url="https://github.com/pytest-dev/pytest-order",
    packages=["pytest_order"],
    license="MIT",
    entry_points={
        "pytest11": [
            "pytest_order = pytest_order.plugin",
        ]
    },
    python_requires=">=3.6",
    install_requires=[
        "pytest>=5.0; python_version < '3.10'",
        "pytest>=6.2.4; python_version >= '3.10'",
    ],
    classifiers=[
        "Development Status :: 4 - Beta",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: MIT License",
        "Operating System :: POSIX",
        "Operating System :: Microsoft :: Windows",
        "Operating System :: MacOS :: MacOS X",
        "Topic :: Software Development :: Testing",
        "Topic :: Software Development :: Quality Assurance",
        "Topic :: Utilities",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
        "Programming Language :: Python :: 3.11",
        "Programming Language :: Python :: 3.12",
        "Programming Language :: Python :: 3 :: Only",
        "Programming Language :: Python :: Implementation :: CPython",
        "Programming Language :: Python :: Implementation :: PyPy",
    ],
)



from textwrap import dedent

import pytest


def test_relative(item_names_for):
    test_content = """
        import pytest

        @pytest.mark.order(after="test_second")
        def test_third():
            pass

        def test_second():
            pass

        @pytest.mark.order(before="test_second")
        def test_first():
            pass
        """
    assert item_names_for(test_content) == ["test_first", "test_second", "test_third"]


def test_relative2(item_names_for):
    test_content = """
        import pytest

        @pytest.mark.order(after="test_second")
        def test_third():
            pass

        def test_second():
            pass

        @pytest.mark.order(before="test_second")
        def test_first():
            pass

        def test_five():
            pass

        @pytest.mark.order(before="test_five")
        def test_four():
            pass
        """
    assert item_names_for(test_content) == [
        "test_first",
        "test_second",
        "test_third",
        "test_four",
        "test_five",
    ]


def test_relative3(item_names_for):
    test_content = """
        import pytest

        @pytest.mark.order(after="test_second")
        def test_third():
            pass

        def test_second():
            pass

        @pytest.mark.order(before="test_second")
        def test_first():
            pass

        def test_five():
            pass

        @pytest.mark.order(before="test_five")
        def test_four():
            pass
        """
    assert item_names_for(test_content) == [
        "test_first",
        "test_second",
        "test_third",
        "test_four",
        "test_five",
    ]


def test_relative_in_class(item_names_for):
    tests_content = """
        import pytest

        class Test:
            @pytest.mark.order(after="test_b")
            def test_a(self):
                pass

            def test_b(self):
                pass

            def test_c(self):
                pass
        """
    assert item_names_for(tests_content) == [
        "Test::test_b",
        "Test::test_a",
        "Test::test_c",
    ]


def test_relative_in_classes(item_names_for):
    tests_content = """
        import pytest

        class TestA:
            @pytest.mark.order(after="TestB::test_b")
            def test_a(self):
                pass

            @pytest.mark.order(after="test_c")
            def test_b(self):
                pass

            def test_c(self):
                pass

        class TestB:
            @pytest.mark.order(before="TestA::test_c")
            def test_a(self):
                pass

            def test_b(self):
                pass

            def test_c(self):
                pass
        """
    assert item_names_for(tests_content) == [
        "TestB::test_a",
        "TestA::test_c",
        "TestA::test_b",
        "TestB::test_b",
        "TestA::test_a",
        "TestB::test_c",
    ]


@pytest.fixture
def fixture_path(test_path):
    test_path.makepyfile(
        mod1_test=(
            """
            import pytest

            class TestA:
                @pytest.mark.order(after="mod2_test.py::TestB::test_b")
                def test_a(self):
                    pass

                @pytest.mark.order(after="sub/mod3_test.py::test_b")
                def test_b(self):
                    pass

                def test_c(self):
                    pass
            """
        ),
        mod2_test=(
            """
            import pytest

            class TestB:
                @pytest.mark.order(before="mod1_test.py::TestA::test_c")
                def test_a(self):
                    pass

                def test_b(self):
                    pass

                def test_c(self):
                    pass
            """
        ),
    )
    test_path.mkpydir("sub")
    path = test_path.tmpdir.join("sub", "mod3_test.py")
    path.write(
        dedent(
            """
        import pytest

        @pytest.mark.order(before="mod2_test.py::TestB::test_c")
        def test_a():
            pass

        def test_b():
            pass

        def test_c():
            pass
        """
        )
    )
    yield test_path


def test_relative_in_modules(fixture_path):
    result = fixture_path.runpytest("-v")
    result.assert_outcomes(passed=9, failed=0)
    result.stdout.fnmatch_lines(
        [
            "mod2_test.py::TestB::test_a PASSED",
            "mod1_test.py::TestA::test_c PASSED",
            "mod2_test.py::TestB::test_b PASSED",
            "mod1_test.py::TestA::test_a PASSED",
            "sub/mod3_test.py::test_a PASSED",
            "mod2_test.py::TestB::test_c PASSED",
            "sub/mod3_test.py::test_b PASSED",
            "mod1_test.py::TestA::test_b PASSED",
            "sub/mod3_test.py::test_c PASSED",
        ]
    )


def test_false_insert(item_names_for):
    test_content = """
        import pytest

        @pytest.mark.order(after="test_a")
        def test_third():
            pass

        def test_second():
            pass

        @pytest.mark.order(before="test_b")
        def test_first():
            pass
        """
    assert item_names_for(test_content) == ["test_third", "test_second", "test_first"]


def test_mixed_markers1(item_names_for):
    test_content = """
        import pytest

        @pytest.mark.order(2)
        def test_1():
            pass

        @pytest.mark.order(after="test_1")
        def test_2():
            pass

        @pytest.mark.order(1)
        def test_3():
            pass
        """
    assert item_names_for(test_content) == ["test_3", "test_1", "test_2"]


def test_mixed_markers2(item_names_for):
    test_content = """
        import pytest

        @pytest.mark.order(2)
        def test_1():
            pass

        @pytest.mark.order(1)
        def test_2():
            pass

        @pytest.mark.order(before="test_2")
        def test_3():
            pass
        """
    assert item_names_for(test_content) == ["test_3", "test_2", "test_1"]


def test_combined_markers1(item_names_for):
    test_content = """
        import pytest

        @pytest.mark.order(2)
        def test_1():
            pass

        def test_2():
            pass

        @pytest.mark.order(index=1, before="test_2")
        def test_3():
            pass
        """
    assert item_names_for(test_content) == ["test_3", "test_1", "test_2"]


def test_combined_markers2(item_names_for):
    test_content = """
        import pytest

        def test_1():
            pass

        @pytest.mark.order(index=2, before="test_1")
        def test_2():
            pass

        @pytest.mark.order(index=1, before="test_1")
        def test_3():
            pass
        """
    assert item_names_for(test_content) == ["test_3", "test_2", "test_1"]


def test_combined_markers3(item_names_for):
    test_content = """
        import pytest

        def test_1():
            pass

        @pytest.mark.order(index=2, before="test_3")
        def test_2():
            pass

        @pytest.mark.order(index=1, before="test_1")
        def test_3():
            pass
        """
    assert item_names_for(test_content) == ["test_2", "test_3", "test_1"]


def test_mixed_markers4(item_names_for):
    test_content = """
        import pytest

        @pytest.mark.order(2)
        def test_1():
            pass

        @pytest.mark.order(index=1, after="test_3")
        def test_2():
            pass

        def test_3():
            pass
        """
    assert item_names_for(test_content) == ["test_3", "test_2", "test_1"]


def test_multiple_markers_in_same_test(item_names_for):
    test_content = """
        import pytest

        @pytest.mark.order(after=["test_3", "test_4", "test_5"])
        def test_1():
            pass

        def test_2():
            pass

        def test_3():
            pass

        @pytest.mark.order(before=["test_3", "test_2"])
        def test_4():
            pass

        def test_5():
            pass
        """
    assert item_names_for(test_content) == [
        "test_4",
        "test_2",
        "test_3",
        "test_5",
        "test_1",
    ]


def test_dependency_after_unknown_test(item_names_for, capsys):
    test_content = """
        import pytest

        @pytest.mark.order(after="some_module.py::test_2")
        def test_1():
            pass

        def test_2():
            pass
        """
    assert item_names_for(test_content) == ["test_1", "test_2"]
    out, err = capsys.readouterr()
    warning = (
        "cannot execute 'test_1' relative to others: 'some_module.py::test_2' "
        "- ignoring the marker"
    )
    assert warning in out


def test_dependency_before_unknown_test(item_names_for, capsys):
    test_content = """
        import pytest

        def test_1():
            pass

        @pytest.mark.order(before="test_4")
        def test_2():
            pass

        def test_3():
            pass
        """
    assert item_names_for(test_content) == ["test_1", "test_2", "test_3"]
    out, err = capsys.readouterr()
    warning = (
        "cannot execute 'test_2' relative to others: 'test_4' " "- ignoring the marker"
    )
    assert warning in out


def test_dependency_in_class_before_unknown_test(item_names_for, capsys):
    test_content = """
        import pytest

        class Test:
            def test_1(self):
                pass

            @pytest.mark.order(before="test_4")
            def test_2(self):
                pass

            def test_3(self):
                pass
        """
    assert item_names_for(test_content) == [
        "Test::test_1",
        "Test::test_2",
        "Test::test_3",
    ]
    out, err = capsys.readouterr()
    warning = (
        "cannot execute 'test_2' relative to others: 'test_4' " "- ignoring the marker"
    )
    assert warning in out


def test_dependency_loop(item_names_for, capsys):
    test_content = """
        import pytest

        @pytest.mark.order(after="test_3")
        def test_1():
            pass

        @pytest.mark.order(1)
        def test_2():
            pass

        @pytest.mark.order(before="test_1")
        def test_3():
            pass
        """
    assert item_names_for(test_content) == ["test_2", "test_1", "test_3"]
    out, err = capsys.readouterr()
    warning = (
        "cannot execute test relative to others: " "test_dependency_loop.py::test_3"
    )
    assert warning in out


def test_dependency_on_parametrized_test(item_names_for):
    test_content = """
        import pytest

        @pytest.mark.order(after="test_2")
        def test_1():
            pass

        @pytest.mark.parametrize("arg", ["aaaaa", "bbbbb", "ccccc", "ddddd"])
        def test_2(arg):
            pass

        @pytest.mark.order(before="test_2")
        def test_3():
            pass
        """
    assert item_names_for(test_content) == [
        "test_3",
        "test_2[aaaaa]",
        "test_2[bbbbb]",
        "test_2[ccccc]",
        "test_2[ddddd]",
        "test_1",
    ]
